<script>
export default {
  // eslint-disable-next-line vue/no-reserved-component-names
  name: 'Embed',
}
</script>

<template>
  <div id="app">
    <router-view />
  </div>
</template>

<style lang="scss">
@import 'scss/_index.scss';
</style>
