@import "bulma-preset-overrides";

/*
ELEMENTS -------------------------------------------------------------------------------------------
Styles related to first-class elements
*/

html {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  background-color: $app-background-color;
}

code {
  white-space: pre-line;
  word-wrap: break-word;
}
