export const PIPELINE_INTERVAL_OPTIONS = {
  '@once': 'Once (Manual)',
  '@hourly': 'Hourly',
  '@daily': 'Daily',
  '@weekly': 'Weekly',
  '@monthly': 'Monthly',
  '@yearly': 'Yearly',
  '@other': 'Other',
}
export const TRANSFORM_OPTIONS = ['run', 'only', 'skip']
